@@ -6,6 +6,11 @@
 
 # ifdef IS_IN_rtld
 #  include <dl-sysdep.h>
+#  ifndef RTLD_PRIVATE_ERRNO
+#   error "dl-sysdep.h must define RTLD_PRIVATE_ERRNO!"
+#  endif
+# else
+#  define RTLD_PRIVATE_ERRNO	0
 # endif
 
 # if RTLD_PRIVATE_ERRNO
@@ -17,7 +22,7 @@
 #  define errno rtld_errno
 extern int rtld_errno attribute_hidden;
 
-# else
+# elif !defined NOT_IN_libc || defined IN_LIB
 
 #  include <tls.h>
 
@@ -29,7 +34,7 @@ extern int rtld_errno attribute_hidden;
 #  endif
 extern __thread int errno attribute_tls_model_ie;
 
-# endif	/* RTLD_PRIVATE_ERRNO */
+# endif	/* !NOT_IN_libc || IN_LIB */
 
 # define __set_errno(val) (errno = (val))
 
