@@ -24,9 +24,6 @@ common_objpfx=$1
 test_program_prefix=$2
 objpfx=$3
 
-LC_ALL=C
-export LC_ALL
-
 # Create the domain directories.
 mkdir -p ${objpfx}domaindir/de_DE/LC_MESSAGES
 mkdir -p ${objpfx}domaindir/fr_FR/LC_MESSAGES
@@ -34,11 +31,6 @@ mkdir -p ${objpfx}domaindir/fr_FR/LC_MESSAGES
 msgfmt -o ${objpfx}domaindir/de_DE/LC_MESSAGES/multithread.mo tst-gettext4-de.po
 msgfmt -o ${objpfx}domaindir/fr_FR/LC_MESSAGES/multithread.mo tst-gettext4-fr.po
 
-GCONV_PATH=${common_objpfx}iconvdata
-export GCONV_PATH
-LOCPATH=${common_objpfx}localedata
-export LOCPATH
-
 ${test_program_prefix} ${objpfx}tst-gettext4 > ${objpfx}tst-gettext4.out
 
 exit $?
