@@ -21,7 +21,7 @@
 # define FPU_CONTROL _FPU_IEEE
 #endif
 
-#include "test-fpucw.c"
+#include <test-fpucw.c>
 
 /* Preempt the library's definition of `__fpu_control'.  */
 fpu_control_t __fpu_control = FPU_CONTROL;
