@@ -1,5 +1,4 @@
-/* Copyright (C) 1995, 1996, 1997, 1998, 2000, 2008
-   Free Software Foundation, Inc.
+/* Copyright (C) 1995-2014 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -96,7 +95,7 @@ getfsfile (name)
 
 
 void
-endfsent ()
+endfsent (void)
 {
   struct fstab_state *state;
 
