@@ -1,10 +1,11 @@
-/* Copyright (c) 1998, 2000, 2003-2006, 2007 Free Software Foundation, Inc.
+/* Copyright (c) 1998-2014 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Thorsten Kukuk <kukuk@suse.de>, 1998.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2 as
-   published by the Free Software Foundation.
+   it under the terms of the GNU General Public License as published
+   by the Free Software Foundation; version 2 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 #include <ctype.h>
 #include <errno.h>
@@ -42,7 +42,8 @@ const char *const dbnames[lastdb] =
   [pwddb] = "passwd",
   [grpdb] = "group",
   [hstdb] = "hosts",
-  [servdb] = "services"
+  [servdb] = "services",
+  [netgrdb] = "netgroup"
 };
 
 
@@ -139,7 +140,8 @@ nscd_parse_file (const char *fname, struct database_dyn dbs[lastdb])
 	{
 	  int idx = find_db (arg1);
 	  if (idx >= 0)
-	    dbs[idx].suggested_module = atol (arg2);
+	    dbs[idx].suggested_module
+	      = atol (arg2) ?: DEFAULT_SUGGESTED_MODULE;
 	}
       else if (strcmp (entry, "enable-cache") == 0)
 	{
@@ -167,7 +169,7 @@ nscd_parse_file (const char *fname, struct database_dyn dbs[lastdb])
 	{
 	  int idx = find_db (arg1);
 	  if (idx >= 0)
-	    dbs[idx].max_db_size = atol (arg2);
+	    dbs[idx].max_db_size = atol (arg2) ?: DEFAULT_MAX_DB_SIZE;
 	}
       else if (strcmp (entry, "logfile") == 0)
 	set_logfile (arg1);
@@ -187,17 +189,17 @@ nscd_parse_file (const char *fname, struct database_dyn dbs[lastdb])
 	  max_nthreads = MAX (atol (arg1), lastdb);
 	}
       else if (strcmp (entry, "server-user") == 0)
-        {
-          if (!arg1)
-            error (0, 0, _("Must specify user name for server-user option"));
-          else
-            server_user = xstrdup (arg1);
-        }
+	{
+	  if (!arg1)
+	    error (0, 0, _("Must specify user name for server-user option"));
+	  else
+	    server_user = xstrdup (arg1);
+	}
       else if (strcmp (entry, "stat-user") == 0)
-        {
-          if (arg1 == NULL)
-            error (0, 0, _("Must specify user name for stat-user option"));
-          else
+	{
+	  if (arg1 == NULL)
+	    error (0, 0, _("Must specify user name for stat-user option"));
+	  else
 	    {
 	      stat_user = xstrdup (arg1);
 
@@ -205,7 +207,7 @@ nscd_parse_file (const char *fname, struct database_dyn dbs[lastdb])
 	      if (pw != NULL)
 		stat_uid = pw->pw_uid;
 	    }
-        }
+	}
       else if (strcmp (entry, "persistent") == 0)
 	{
 	  int idx = find_db (arg1);
@@ -234,13 +236,11 @@ nscd_parse_file (const char *fname, struct database_dyn dbs[lastdb])
 	    reload_count = UINT_MAX;
 	  else
 	    {
-	      unsigned int count = strtoul (arg1, NULL, 0);
+	      unsigned long int count = strtoul (arg1, NULL, 0);
 	      if (count > UINT8_MAX - 1)
 		reload_count = UINT_MAX;
-	      else if (count >= 0)
-	    reload_count = count;
 	      else
-		error (0, 0, _("invalid value for 'reload-count': %u"), count);
+		reload_count = count;
 	    }
 	}
       else if (strcmp (entry, "paranoia") == 0)
@@ -255,7 +255,7 @@ nscd_parse_file (const char *fname, struct database_dyn dbs[lastdb])
 	  if (arg1 != NULL)
 	    restart_interval = atol (arg1);
 	  else
-            error (0, 0, _("Must specify value for restart-interval option"));
+	    error (0, 0, _("Must specify value for restart-interval option"));
 	}
       else if (strcmp (entry, "auto-propagate") == 0)
 	{
