@@ -1,5 +1,4 @@
-/* Copyright (C) 1998-2000, 2002-2005, 2006, 2007
-   Free Software Foundation, Inc.
+/* Copyright (C) 1998-2014 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Thorsten Kukuk <kukuk@uni-paderborn.de>, 1998.
 
@@ -14,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
-   License along with the GNU C Library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307 USA.  */
+   License along with the GNU C Library; if not, see
+   <http://www.gnu.org/licenses/>.  */
 
 #include <alloca.h>
 #include <assert.h>
@@ -32,7 +30,7 @@
 #include <sys/uio.h>
 #include <sys/un.h>
 #include <not-cancel.h>
-#include <stdio-common/_itoa.h>
+#include <_itoa.h>
 
 #include "nscd-client.h"
 #include "nscd_proto.h"
@@ -107,7 +105,8 @@ nscd_getgr_r (const char *key, size_t keylen, request_type type,
 
   if (mapped != NO_MAPPING)
     {
-      struct datahead *found = __nscd_cache_search (type, key, keylen, mapped);
+      struct datahead *found = __nscd_cache_search (type, key, keylen, mapped,
+						    sizeof gr_resp);
       if (found != NULL)
 	{
 	  len = (const uint32_t *) (&found->data[0].grdata + 1);
@@ -144,7 +143,7 @@ nscd_getgr_r (const char *key, size_t keylen, request_type type,
   /* No value found so far.  */
   *result = NULL;
 
-  if (__builtin_expect (gr_resp.found == -1, 0))
+  if (__glibc_unlikely (gr_resp.found == -1))
     {
       /* The daemon does not cache this database.  */
       __nss_not_use_nscd_group = 1;
@@ -165,7 +164,7 @@ nscd_getgr_r (const char *key, size_t keylen, request_type type,
 	       & (__alignof__ (char *) - 1));
       total_len = (align + (1 + gr_resp.gr_mem_cnt) * sizeof (char *)
 		   + gr_resp.gr_name_len + gr_resp.gr_passwd_len);
-      if (__builtin_expect (buflen < total_len, 0))
+      if (__glibc_unlikely (buflen < total_len))
 	{
 	no_room:
 	  __set_errno (ERANGE);
@@ -190,26 +189,37 @@ nscd_getgr_r (const char *key, size_t keylen, request_type type,
       /* Read the length information, group name, and password.  */
       if (gr_name == NULL)
 	{
-	  /* Allocate array to store lengths.  */
-	  if (lensize == 0)
+	  /* Handle a simple, usual case: no group members.  */
+	  if (__glibc_likely (gr_resp.gr_mem_cnt == 0))
 	    {
-	      lensize = gr_resp.gr_mem_cnt * sizeof (uint32_t);
-	      len = (uint32_t *) alloca (lensize);
+	      size_t n = gr_resp.gr_name_len + gr_resp.gr_passwd_len;
+	      if (__builtin_expect (__readall (sock, resultbuf->gr_name, n)
+				    != (ssize_t) n, 0))
+		goto out_close;
 	    }
-	  else if (gr_resp.gr_mem_cnt * sizeof (uint32_t) > lensize)
-	    len = extend_alloca (len, lensize,
-				 gr_resp.gr_mem_cnt * sizeof (uint32_t));
+	  else
+	    {
+	      /* Allocate array to store lengths.  */
+	      if (lensize == 0)
+		{
+		  lensize = gr_resp.gr_mem_cnt * sizeof (uint32_t);
+		  len = (uint32_t *) alloca (lensize);
+		}
+	      else if (gr_resp.gr_mem_cnt * sizeof (uint32_t) > lensize)
+		len = extend_alloca (len, lensize,
+				     gr_resp.gr_mem_cnt * sizeof (uint32_t));
 
-	  vec[0].iov_base = (void *) len;
-	  vec[0].iov_len = gr_resp.gr_mem_cnt * sizeof (uint32_t);
-	  vec[1].iov_base = resultbuf->gr_name;
-	  vec[1].iov_len = gr_resp.gr_name_len + gr_resp.gr_passwd_len;
-	  total_len = vec[0].iov_len + vec[1].iov_len;
+	      vec[0].iov_base = (void *) len;
+	      vec[0].iov_len = gr_resp.gr_mem_cnt * sizeof (uint32_t);
+	      vec[1].iov_base = resultbuf->gr_name;
+	      vec[1].iov_len = gr_resp.gr_name_len + gr_resp.gr_passwd_len;
+	      total_len = vec[0].iov_len + vec[1].iov_len;
 
-	  /* Get this data.  */
-	  size_t n = __readvall (sock, vec, 2);
-	  if (__builtin_expect (n != total_len, 0))
-	    goto out_close;
+	      /* Get this data.  */
+	      size_t n = __readvall (sock, vec, 2);
+	      if (__glibc_unlikely (n != total_len))
+		goto out_close;
+	    }
 	}
       else
 	/* We already have the data.  Just copy the group name and
@@ -229,7 +239,7 @@ nscd_getgr_r (const char *key, size_t keylen, request_type type,
 	  p += len[cnt];
 	}
 
-      if (__builtin_expect (gr_name + gr_name_len + total_len > recend, 0))
+      if (__glibc_unlikely (gr_name + gr_name_len + total_len > recend))
 	{
 	  /* len array might contain garbage during nscd GC cycle,
 	     retry rather than fail in that case.  */
@@ -237,7 +247,7 @@ nscd_getgr_r (const char *key, size_t keylen, request_type type,
 	    retval = -2;
 	  goto out_close;
 	}
-      if (__builtin_expect (total_len > buflen, 0))
+      if (__glibc_unlikely (total_len > buflen))
 	{
 	  /* len array might contain garbage during nscd GC cycle,
 	     retry rather than fail in that case.  */
@@ -251,10 +261,13 @@ nscd_getgr_r (const char *key, size_t keylen, request_type type,
 	}
 
       retval = 0;
+
+      /* If there are no group members TOTAL_LEN is zero.  */
       if (gr_name == NULL)
 	{
-	  size_t n = __readall (sock, resultbuf->gr_mem[0], total_len);
-	  if (__builtin_expect (n != total_len, 0))
+	  if (total_len > 0
+	      && __builtin_expect (__readall (sock, resultbuf->gr_mem[0],
+					      total_len) != total_len, 0))
 	    {
 	      /* The `errno' to some value != ERANGE.  */
 	      __set_errno (ENOENT);
@@ -272,9 +285,9 @@ nscd_getgr_r (const char *key, size_t keylen, request_type type,
 	  if (resultbuf->gr_name[gr_name_len - 1] != '\0'
 	      || resultbuf->gr_passwd[gr_resp.gr_passwd_len - 1] != '\0'
 	      || ({for (cnt = 0; cnt < gr_resp.gr_mem_cnt; ++cnt)
-		     if (resultbuf->gr_mem[cnt][len[cnt] - 1] != '\0')
-		       break;
-	  	   cnt < gr_resp.gr_mem_cnt; }))
+		    if (resultbuf->gr_mem[cnt][len[cnt] - 1] != '\0')
+		      break;
+		  cnt < gr_resp.gr_mem_cnt; }))
 	    {
 	      /* We cannot use the database.  */
 	      retval = mapped->head->gc_cycle != gc_cycle ? -2 : -1;
@@ -286,8 +299,8 @@ nscd_getgr_r (const char *key, size_t keylen, request_type type,
     }
   else
     {
-      /* The `errno' to some value != ERANGE.  */
-      __set_errno (ENOENT);
+      /* Set errno to 0 to indicate no error, just no found record.  */
+      __set_errno (0);
       /* Even though we have not found anything, the result is zero.  */
       retval = 0;
     }
